// cristinel ababei; 2017-2020; copyleft :-)
//
// Description:
// this is a simple Arduino project which uses a Bluetooth HC-06 module
// to receive/send messages from/to "an Android device OR host PC"; it receives messages
// from Android device and prints them via HW Serial to terminal on PC; 
// it takes from Serial what we type in terminal and sends via HC-06 to
// the Android device; Setup() has the option to change the name of the
// HC-06 module as well - needed to be done once only, after which you should
// pair your Android device with the newly named HC-06;
// to use this sketch you need also the Android project:
// Example1BTString/ from the implementation2_codes/
//
// Credits:
// this project is adapted from:
// https://causeyourestuck.io/2015/12/14/communication-between-android-and-hc-06-module/

#include <SoftwareSerial.h>


#define DEBUG true

// initialize communication port to HC-06 BT module;
SoftwareSerial hc06_sw_serial(11, 10); // RX | TX

 
void setup()  
{
  // (1) default HW serial used usually for debug messages sent back to PC;
  Serial.begin(9600);
  
  // (2) initialize SW serial link to HC-06; baudrate for communicating 
  // with the HC-06 is 9600;
  hc06_sw_serial.begin(9600); 
  
  // (3) send three AT commands to the HC-06 module; the third one is 
  // the one that changes its name to "HC06-CHAT"; 
  // Note: you have to do this once only: compile and program Arduino; see
  // confirmation that name of HC-06 is changed, then, comment out this part,
  // recompile, program Arduino, open also Android device, pair with newly
  // named HC-06 and verify operation of Android device <-> Arduino comm.
  send_command_to_hc06("AT\r\n",1000,DEBUG); 
  send_command_to_hc06("AT+VERSION\r\n",1000,DEBUG); 
  send_command_to_hc06("AT+NAME=HC06-CHAT-CRIS\r\n",1000,DEBUG);
}


void loop()
{
  // (1) check if a message has been received via HC-06 from the Android device
  // that this HC-06 is paired with;
  String msg = get_message_from_hc06();
  if (msg!="") {
    Serial.println(msg);
  }

  // (2) send the message to the HW Serial Monitor to the HC-06
  if (Serial.available()) {
    hc06_sw_serial.write(Serial.read());
  }
  // or send some custom string; 
  // hc06_sw_serial.println("your message");

  // (3) short break;
  delay(10);
}


///////////////////////////////////////////////////////////////////////////////
//
// functions related to handling the HC-06 bluetooth module;
//
///////////////////////////////////////////////////////////////////////////////

// Name: send_command_to_hc06
// Description: Function used to send AT commands to HC-06.
// Params: 
//    command - the data/command to send; 
//    timeout - the time to wait for a response; 
//    debug - print to Serial window? (true = yes, false = no);
// Returns: The response from the HC-06 bluetooth module (if there is a reponse);
String send_command_to_hc06(String command, const int timeout, boolean debug)
{
  String response = ""; 
  hc06_sw_serial.print(command); // send the read character to the hc-06;
  
  long int time = millis();   
  while ( (time+timeout) > millis()) {
    while (hc06_sw_serial.available()) {      
      // hc-06 module has data; so display its output to the serial window; 
      char c = hc06_sw_serial.read(); // read the next character
      response+=c;
    }  
  }
  
  if (debug) {
    Serial.print(response);
    Serial.print("\n");
  }   
  return response;
}


String get_message_from_hc06()
{
  String msg = "";
  char a;
  while (hc06_sw_serial.available()) {
      a = hc06_sw_serial.read();
      msg+=String(a);
  }
  return msg;
}
