// cristinel ababei; 
//
// Description:
// this is a simple Arduino project which uses a temperature/humidity
// sensor to read temperature and humidity; temperature and humidity
// are sent to the host PC via Bluetooth using an HC-06 module;
// in addition, in the beginning only, it also changes the name
// of the HC-06 module to make it unique to my situation; this
// helps in a class-room scenario where multiple modules may be on
// at the same time;

#include <dht.h>
#include <SoftwareSerial.h>

#define DEBUG true

// (1) DHT22
// digital pin 6 of the Arduino board is connected to pin DTA of the 
// DHT22 sensor, like in this diagram
//  ________________
// |                |
// |  DHT22 Sensor  | Front view (see thru) of DHT22 sensor
// |  1   2   3  4  |
// | VDD DTA NC GND |
//  ----------------
//   |   |   |   |
//   |   |   |   |
//       ^is connected to digital pin 6 of Arduino board
//   ^is connected to 5V of Arduino
//               ^is connected to a GND pin of Arduino
//           ^is left disconnected (i.e., Not-Connected)
dht DHT;
#define DHT22_PIN 6

// (2) HC-06 BT module
// connect the HC-06 TX to Arduino pin 2 RX;
// connect the HC-06 RX to Arduino pin 3 TX through a voltage divider created with 
// three identical resistors;
SoftwareSerial hc06_serial(2, 3); // RX | TX


void setup()
{
  // (1) after the Arduino is programmed, we can use the Arduino's serial
  // terminal as a debugging interface; for example, we can print to it
  // anything that the Arduino sends to the bluetooth module;
  // in our case, we'll use it to only display the response from
  // the HC-06 module when we configure it to change its name;
  Serial.begin(9600);
  
  // (2) initialize serial link to HC-06; baudrate for communicating 
  // with the HC-06 is 9600;
  hc06_serial.begin(9600); 
  // send three AT commands to the HC-06 module; the third one is 
  // the one that changes its name to "cristinel";
  sendCommand("AT",1000,DEBUG); 
  sendCommand("AT+VERSION",1000,DEBUG); 
  sendCommand("AT+NAMEcris",1000,DEBUG);
  // directly send to the HC-06 module the header of the humidity
  // and temperatue data, which will be send by the HC-06 module via 
  // bluetooth;
  hc06_serial.println("DHT22 DATA ACQUISITION PROGRAM");
  hc06_serial.println();
  hc06_serial.println("Status \tHumidity (%) \tTemperature (^C)");
  delay(1000);
}


void loop()
{
  // (1) read data from the DHT22 sensor; sanity check that
  // all is ok and print status;
  int chk = DHT.read22( DHT22_PIN);
  switch ( chk) {
    case DHTLIB_OK:  
  		hc06_serial.print("OK \t"); 
  		break;
    case DHTLIB_ERROR_CHECKSUM: 
  		hc06_serial.print("Checksum error,\t"); 
  		break;
    case DHTLIB_ERROR_TIMEOUT: 
  		hc06_serial.print("Time out error \t"); 
  		break;
    default: 
  		hc06_serial.print("Unknown error \t"); 
  		break;
  }
  
  // (2) send the humidity and temperature data to the HC-06 module via 
  // software serial connection; 
  hc06_serial.print(DHT.humidity, 1);
  hc06_serial.print(" \t");
  hc06_serial.println(DHT.temperature, 1);

  // (3) wait for about 2 seconds;
  delay(2000);
}

///////////////////////////////////////////////////////////////////////////////
//
// functions related to handling the HC-06 bluetooth module;
//
///////////////////////////////////////////////////////////////////////////////

// Name: sendCommand
// Description: Function used to send AT commands to HC-06.
// Params: 
//    command - the data/command to send; 
//    timeout - the time to wait for a response; 
//    debug - print to Serial window? (true = yes, false = no);
// Returns: The response from the HC-06 bluetooth module (if there is a reponse);
String sendCommand(String command, const int timeout, boolean debug)
{
    String response = "";
           
    hc06_serial.print(command); // send the read character to the hc-06;
    
    long int time = millis();   
    while ( (time+timeout) > millis())
    {
      while (hc06_serial.available())
      {      
        // hc-06 module has data; so display its output to the serial window; 
        char c = hc06_serial.read(); // read the next character
        response+=c;
      }  
    }
    
    if (debug) {
      Serial.print(response);
      Serial.print("\n");
    }   
    return response;
}
